
import tempfile

import pytest
import responses

try:
    from unittest.mock import patch
except ImportError:
    from mock import patch

from instabot.api.config import API_URL, SIG_KEY_VERSION
from instabot import utils

from .test_bot import TestBot
from .test_variables import (TEST_CAPTION_ITEM, TEST_COMMENT_ITEM,
                             TEST_PHOTO_ITEM, TEST_SEARCH_USERNAME_ITEM,
                             TEST_USER_ITEM, TEST_USERNAME_INFO_ITEM,
                             TEST_TIMELINE_PHOTO_ITEM, TEST_USER_TAG_ITEM,
                             TEST_MEDIA_LIKER, TEST_FOLLOWER_ITEM, TEST_FOLLOWING_ITEM,
                             TEST_COMMENT_LIKER_ITEM, TEST_LOCATION_ITEM,
                             TEST_MOST_RECENT_INVITER_ITEM, TEST_INBOX_THREAD_ITEM)


class TestBotGet(TestBot):
    @responses.activate
    def test_get_media_owner(self):
        media_id = 1234

        responses.add(
            responses.GET, "{api_url}media/{media_id}/info/".format(
                api_url=API_URL, media_id=media_id),
            json={
                "auto_load_more_enabled": True,
                "num_results": 1,
                "status": "ok",
                "more_available": False,
                "items": [TEST_PHOTO_ITEM]
            }, status=200)
        # responses.add(
        #     responses.POST, "{api_url}media/{media_id}/info/".format(
        #     api_url=API_URL, media_id=media_id),
        #     json={"status": "ok"}, status=200)

        owner = self.bot.get_media_owner(media_id)

        assert owner == str(TEST_PHOTO_ITEM["user"]["pk"])

        # owner = self.bot.get_media_owner(media_id)

        # assert owner is False

    @responses.activate
    def test_get_media_info(self):
        media_id = 1234

        responses.add(
            responses.GET, "{api_url}media/{media_id}/info/".format(
                api_url=API_URL, media_id=media_id),
            json={
                "auto_load_more_enabled": True,
                "num_results": 1,
                "status": "ok",
                "more_available": False,
                "items": [TEST_PHOTO_ITEM]
            }, status=200)
        # responses.add(
        #     responses.POST, "{api_url}media/{media_id}/info/".format(
        #     api_url=API_URL, media_id=media_id),
        #     json={"status": "ok"}, status=200)

        expected_result = {}
        for key in TEST_PHOTO_ITEM:
            expected_result[key] = TEST_PHOTO_ITEM[key]

        result = self.bot.get_media_info(media_id)

        assert result[0] == expected_result

    @responses.activate
    def test_get_popular_medias(self):
        results = 5
        responses.add(
            responses.GET, "{api_url}feed/popular/?people_teaser_supported=1&rank_token={rank_token}&ranked_content=true&".format(
                api_url=API_URL, rank_token=self.bot.api.rank_token),
            json={
                "auto_load_more_enabled": True,
                "num_results": results,
                "status": "ok",
                "more_available": False,
                "items": [TEST_PHOTO_ITEM for _ in range(results)]
            }, status=200)

        medias = self.bot.get_popular_medias()

        assert medias == [str(TEST_PHOTO_ITEM["pk"]) for _ in range(results)]
        assert len(medias) == results

    @responses.activate
    def test_get_timeline_medias(self):
        self.bot.max_likes_to_like = TEST_PHOTO_ITEM['like_count'] + 1
        results = 8
        responses.add(
            responses.GET, "{api_url}feed/timeline/".format(api_url=API_URL),
            json={
                "auto_load_more_enabled": True,
                "num_results": results,
                "is_direct_v2_enabled": True,
                "status": "ok",
                "next_max_id": None,
                "more_available": False,
                "feed_items": [TEST_TIMELINE_PHOTO_ITEM for _ in range(results)]
            }, status=200)
        responses.add(
            responses.GET, "{api_url}feed/timeline/".format(api_url=API_URL),
            json={
                "status": "fail"
            }, status=400)

        medias = self.bot.get_timeline_medias()

        assert medias == [TEST_PHOTO_ITEM["pk"] for _ in range(results)]
        assert len(medias) == results

        medias = self.bot.get_timeline_medias()

        assert medias == []
        assert len(medias) == 0

    @responses.activate
    def test_get_timeline_users(self):
        results = 8
        responses.add(
            responses.GET, "{api_url}feed/timeline/".format(api_url=API_URL),
            json={
                "auto_load_more_enabled": True,
                "num_results": results,
                "is_direct_v2_enabled": True,
                "status": "ok",
                "next_max_id": None,
                "more_available": False,
                "feed_items": [TEST_TIMELINE_PHOTO_ITEM for _ in range(results)]
            }, status=200)
        responses.add(
            responses.GET, "{api_url}feed/timeline/".format(api_url=API_URL),
            json={
                "status": "fail"
            }, status=400)

        users = self.bot.get_timeline_users()

        assert users == [str(TEST_TIMELINE_PHOTO_ITEM["media_or_ad"]["user"]["pk"]) for _ in range(results)]
        assert len(users) == results

        users = self.bot.get_timeline_users()

        assert users == []
        assert len(users) == 0

    @responses.activate
    def test_get_your_medias(self):
        results = 5
        my_test_photo_item = TEST_PHOTO_ITEM.copy()
        my_test_photo_item['user']['pk'] = self.USER_ID
        response_data = {
            "auto_load_more_enabled": True,
            "num_results": results,
            "status": "ok",
            "more_available": False,
            "items": [my_test_photo_item for _ in range(results)]
        }
        responses.add(
            responses.GET, '{api_url}feed/user/{user_id}/'.format(
                api_url=API_URL, user_id=self.bot.user_id, rank_token=self.bot.api.rank_token),
            json=response_data, status=200)

        medias = self.bot.get_your_medias()

        assert medias == [my_test_photo_item["pk"] for _ in range(results)]
        assert len(medias) == results

        medias = self.bot.get_your_medias(as_dict=True)

        assert medias == response_data['items']
        assert len(medias) == results

    @responses.activate
    @pytest.mark.parametrize('user_id', [
        1234567890, '1234567890'
    ])
    def test_get_user_medias(self, user_id):
        results = 4
        my_test_photo_item = TEST_PHOTO_ITEM.copy()
        my_test_photo_item['user']['pk'] = user_id
        my_test_photo_items = []
        for _ in range(results):
            my_test_photo_items.append(my_test_photo_item.copy())
        expect_filtered = 0
        my_test_photo_items[1]["has_liked"] = True
        expect_filtered += 1
        my_test_photo_items[2]["like_count"] = self.bot.max_likes_to_like + 1
        expect_filtered += 1
        my_test_photo_items[3]["like_count"] = self.bot.max_likes_to_like - 1
        expect_filtered += 1
        response_data = {
            "auto_load_more_enabled": True,
            "num_results": results,
            "status": "ok",
            "more_available": False,
            "items": my_test_photo_items
        }
        responses.add(
            responses.GET, '{api_url}feed/user/{user_id}/'.format(
                api_url=API_URL, user_id=user_id, rank_token=self.bot.api.rank_token),
            json=response_data, status=200)

        # no need to test is_comment=True because there's no item 'comments' in
        # user feed object returned by `feed/user/{user_id}/` API call.

        medias = self.bot.get_user_medias(user_id, filtration=False, is_comment=False)
        assert medias == [test_photo_item["pk"] for test_photo_item in my_test_photo_items]
        assert len(medias) == results

        medias = self.bot.get_user_medias(user_id, filtration=True, is_comment=False)
        assert medias == [test_photo_item["pk"] for test_photo_item in my_test_photo_items if (not test_photo_item["has_liked"] and test_photo_item["like_count"] < self.bot.max_likes_to_like and test_photo_item["like_count"] > self.bot.min_likes_to_like)]
        assert len(medias) == results - expect_filtered

    @responses.activate
    def test_get_archived_medias(self):
        results = 5
        my_test_photo_item = TEST_PHOTO_ITEM.copy()
        my_test_photo_item['user']['pk'] = self.USER_ID
        response_data = {
            "auto_load_more_enabled": True,
            "num_results": results,
            "status": "ok",
            "more_available": False,
            "items": [my_test_photo_item for _ in range(results)]
        }
        responses.add(
            responses.GET, '{api_url}feed/only_me_feed/?rank_token={rank_token}&ranked_content=true&'.format(
                api_url=API_URL, rank_token=self.bot.api.rank_token),
            json=response_data, status=200)

        medias = self.bot.get_archived_medias()

        assert medias == [my_test_photo_item["pk"] for _ in range(results)]
        assert len(medias) == results

        medias = self.bot.get_archived_medias(as_dict=True)

        assert medias == response_data['items']
        assert len(medias) == results

    @responses.activate
    def test_search_users(self):
        results = 5
        query = "test"
        my_test_user_item = TEST_USER_ITEM
        response_data = {
            "has_more": True,
            "num_results": results,
            "rank_token": self.bot.api.rank_token,
            "status": "ok",
            "users": [my_test_user_item for _ in range(results)]
        }
        responses.add(
            responses.GET, '{api_url}users/search/?ig_sig_key_version={sig_key}&is_typeahead=true&query={query}&rank_token={rank_token}'.format(
                api_url=API_URL, rank_token=self.bot.api.rank_token, query=query, sig_key=SIG_KEY_VERSION), json=response_data, status=200)

        medias = self.bot.search_users(query)

        assert medias == [str(my_test_user_item["pk"]) for _ in range(results)]
        assert len(medias) == results

    @responses.activate
    def test_search_users_failed(self):
        query = "test"
        response_data = {'status': 'fail'}
        responses.add(
            responses.GET, '{api_url}users/search/?ig_sig_key_version={sig_key}&is_typeahead=true&query={query}&rank_token={rank_token}'.format(
                api_url=API_URL, rank_token=self.bot.api.rank_token, query=query, sig_key=SIG_KEY_VERSION), json=response_data, status=200)

        medias = self.bot.search_users(query)

        assert medias == []

    @responses.activate
    def test_get_comments(self):
        results = 5
        response_data = {
            "caption": TEST_CAPTION_ITEM,
            "caption_is_edited": False,
            "comment_count": 4,
            "comment_likes_enabled": True,
            "comments": [TEST_COMMENT_ITEM for _ in range(results)],
            "has_more_comments": False,
            "has_more_headload_comments": False,
            "media_header_display": "none",
            "preview_comments": [],
            "status": "ok"
        }
        media_id = 1234567890
        responses.add(
            responses.GET, '{api_url}media/{media_id}/comments/?'.format(
                api_url=API_URL, media_id=media_id), json=response_data, status=200)

        comments = self.bot.get_media_comments(media_id)
        assert comments == response_data['comments']
        assert len(comments) == results

    @responses.activate
    def test_get_comments_text(self):
        results = 5
        response_data = {
            "caption": TEST_CAPTION_ITEM,
            "caption_is_edited": False,
            "comment_count": 4,
            "comment_likes_enabled": True,
            "comments": [TEST_COMMENT_ITEM for _ in range(results)],
            "has_more_comments": False,
            "has_more_headload_comments": False,
            "media_header_display": "none",
            "preview_comments": [],
            "status": "ok"
        }
        media_id = 1234567890
        responses.add(
            responses.GET, '{api_url}media/{media_id}/comments/?'.format(
                api_url=API_URL, media_id=media_id), json=response_data, status=200)

        comments = self.bot.get_media_comments(media_id, only_text=True)
        expected_result = [comment['text'] for comment in response_data['comments']]

        assert comments == expected_result
        assert len(comments) == results

    @responses.activate
    def test_get_comments_failed(self):
        response_data = {"status": "fail"}
        media_id = 1234567890
        responses.add(
            responses.GET, '{api_url}media/{media_id}/comments/?'.format(
                api_url=API_URL, media_id=media_id), json=response_data, status=200)

        comments = self.bot.get_media_comments(media_id)
        assert comments == []

    @responses.activate
    def test_get_commenters(self):
        results = 5
        response_data = {
            "caption": TEST_CAPTION_ITEM,
            "caption_is_edited": False,
            "comment_count": 4,
            "comment_likes_enabled": True,
            "comments": [TEST_COMMENT_ITEM for _ in range(results)],
            "has_more_comments": False,
            "has_more_headload_comments": False,
            "media_header_display": "none",
            "preview_comments": [],
            "status": "ok"
        }
        media_id = 1234567890
        responses.add(
            responses.GET, '{api_url}media/{media_id}/comments/?'.format(
                api_url=API_URL, media_id=media_id), json=response_data, status=200)

        expected_commenters = [str(TEST_COMMENT_ITEM['user']['pk']) for _ in range(results)]

        commenters = self.bot.get_media_commenters(media_id)
        assert commenters == expected_commenters
        assert len(commenters) == results

    @responses.activate
    def test_get_commenters_failed(self):
        response_data = {"status": "fail"}
        media_id = 1234567890
        responses.add(
            responses.GET, '{api_url}media/{media_id}/comments/?'.format(
                api_url=API_URL, media_id=media_id), json=response_data, status=200)

        expected_commenters = []

        commenters = self.bot.get_media_commenters(media_id)
        assert commenters == expected_commenters

    @pytest.mark.parametrize('url,result', [
        ('https://www.instagram.com/p/BfHrDvCDuzC/', 1713527555896569026),
        ('test', False)
    ])
    def test_get_media_id_from_link_with_wrong_data(self, url, result):
        media_id = self.bot.get_media_id_from_link(url)

        assert result == media_id

    @responses.activate
    @pytest.mark.parametrize('comments', [
        ['comment1', 'comment2', 'comment3'],
        [],
        None
    ])
    def test_get_comment(self, comments):
        fname = tempfile.mkstemp()[1]  # Temporary file
        self.bot.comments_file = utils.file(fname, verbose=True)
        if comments:
            for comment in comments:
                self.bot.comments_file.append(comment)
            assert self.bot.get_comment() in self.bot.comments_file.list
        else:
            assert self.bot.get_comment() == 'Wow!'

    @responses.activate
    @pytest.mark.parametrize('user_id', [
        1234, '1234'
    ])
    def test_get_username_info(self, user_id):
        response_data = {
            'status': 'ok',
            'user': TEST_USERNAME_INFO_ITEM
        }
        expected_result = {}
        for key in TEST_USERNAME_INFO_ITEM:
            expected_result[key] = TEST_USERNAME_INFO_ITEM[key]

        responses.add(
            responses.GET, '{api_url}users/{user_id}/info/'.format(
                api_url=API_URL, user_id=user_id
            ), status=200, json=response_data)

        result = self.bot.get_user_info(user_id)

        assert result == expected_result

    @responses.activate
    @pytest.mark.parametrize('user_id', [
        1234, '1234'
    ])
    @patch('time.sleep', return_value=None)
    def test_get_username_from_user_id(self, patched_time_sleep, user_id):
        response_data = {
            'status': 'ok',
            'user': TEST_USERNAME_INFO_ITEM
        }
        expected_user_id = str(TEST_USERNAME_INFO_ITEM['username'])

        responses.add(
            responses.GET, '{api_url}users/{user_id}/info/'.format(
                api_url=API_URL, user_id=user_id
            ), status=200, json=response_data)

        result = self.bot.get_username_from_user_id(user_id)

        assert result == expected_user_id

    @responses.activate
    @pytest.mark.parametrize('user_id', [
        '123231231231234', 123231231231234
    ])
    @patch('time.sleep', return_value=None)
    def test_get_username_from_user_id_404(self, patched_time_sleep, user_id):
        response_data = {
            'status': 'fail',
            'message': 'User not found'
        }
        responses.add(
            responses.GET, '{api_url}users/{user_id}/info/'.format(
                api_url=API_URL, user_id=user_id
            ), status=404, json=response_data)

        assert not self.bot.get_username_from_user_id(user_id)

    @responses.activate
    @pytest.mark.parametrize('username', [
        '@test', 'test', '1234'
    ])
    @patch('time.sleep', return_value=None)
    def test_get_user_id_from_username(self, patched_time_sleep, username):
        response_data = {
            'status': 'ok',
            'user': TEST_SEARCH_USERNAME_ITEM
        }
        expected_user_id = str(TEST_SEARCH_USERNAME_ITEM['pk'])

        responses.add(
            responses.GET, '{api_url}users/{username}/usernameinfo/'.format(
                api_url=API_URL, username=username
            ), status=200, json=response_data)

        result = self.bot.get_user_id_from_username(username)
        del self.bot._usernames[username]  # Invalidate cache

        assert result == expected_user_id

    @responses.activate
    @pytest.mark.parametrize('username', [
        'usernotfound', 'nottexisteduser', '123231231231234'
    ])
    @patch('time.sleep', return_value=None)
    def test_get_user_id_from_username_404(self, patched_time_sleep, username):
        response_data = {
            'status': 'fail',
            'message': 'User not found'
        }
        responses.add(
            responses.GET, '{api_url}users/{username}/usernameinfo/'.format(
                api_url=API_URL, username=username
            ), status=404, json=response_data)

        assert not self.bot.get_user_id_from_username(username)

    @responses.activate
    @pytest.mark.parametrize('username,url,result', [
        ('@test', 'test', str(TEST_SEARCH_USERNAME_ITEM['pk'])),
        ('test', 'test', str(TEST_SEARCH_USERNAME_ITEM['pk'])),
        ('1234', '1234', '1234'),
        (1234, '1234', '1234')
    ])
    @patch('time.sleep', return_value=None)
    def test_convert_to_user_id(self, patched_time_sleep, username, url, result):
        response_data = {
            'status': 'ok',
            'user': TEST_SEARCH_USERNAME_ITEM
        }
        responses.add(
            responses.GET, '{api_url}users/{username}/usernameinfo/'.format(
                api_url=API_URL, username=url
            ), status=200, json=response_data)

        user_id = self.bot.convert_to_user_id(username)

        assert result == user_id

    @responses.activate
    @pytest.mark.parametrize('user_id', [
        '3998456661', 3998456661
    ])
    def test_get_user_tags_medias(self, user_id):
        results = 8
        responses.add(
            responses.GET, "{api_url}usertags/{user_id}/feed/?rank_token={rank_token}&ranked_content=true&".format(
                api_url=API_URL, user_id=user_id, rank_token=self.bot.api.rank_token),
            json={
                'status': 'ok',
                'num_results': results,
                'auto_load_more_enabled': True,
                'items': [TEST_USER_TAG_ITEM for _ in range(results)],
                'more_available': False,
                'total_count': results,
                'requires_review': False,
                'new_photos': []
            }, status=200)

        medias = self.bot.get_user_tags_medias(user_id)

        assert medias == [str(TEST_USER_TAG_ITEM["pk"]) for _ in range(results)]
        assert len(medias) == results

    @responses.activate
    @pytest.mark.parametrize('hashtag', [
        'hashtag1', 'hashtag2'
    ])
    def test_get_hashtag_medias(self, hashtag):

        results = 5
        my_test_photo_item = TEST_PHOTO_ITEM.copy()
        my_test_photo_item["like_count"] = self.bot.min_likes_to_like + 1
        my_test_photo_items = []
        for _ in range(results):
            my_test_photo_items.append(my_test_photo_item.copy())
        expect_filtered = 0
        my_test_photo_items[1]["has_liked"] = True
        expect_filtered += 1
        my_test_photo_items[2]["like_count"] = self.bot.max_likes_to_like + 1
        expect_filtered += 1
        my_test_photo_items[3]["like_count"] = self.bot.min_likes_to_like - 1
        expect_filtered += 1
        response_data = {
            "auto_load_more_enabled": True,
            "num_results": results,
            "status": "ok",
            "more_available": False,
            "items": my_test_photo_items
        }

        responses.add(
            responses.GET, '{api_url}feed/tag/{hashtag}/?max_id={max_id}&rank_token={rank_token}&ranked_content=true&'.format(
                api_url=API_URL, hashtag=hashtag, max_id='',
                rank_token=self.bot.api.rank_token),
            json=response_data, status=200)

        medias = self.bot.get_hashtag_medias(hashtag, filtration=False)
        assert medias == [test_photo_item["pk"] for test_photo_item in my_test_photo_items]
        assert len(medias) == results

        medias = self.bot.get_hashtag_medias(hashtag, filtration=True)
        assert medias == [test_photo_item["pk"] for test_photo_item in my_test_photo_items if (not test_photo_item["has_liked"] and test_photo_item["like_count"] < self.bot.max_likes_to_like and test_photo_item["like_count"] > self.bot.min_likes_to_like)]
        assert len(medias) == results - expect_filtered

    @responses.activate
    @pytest.mark.parametrize('hashtag', [
        'hashtag1', 'hashtag2'
    ])
    def test_get_total_hashtag_medias(self, hashtag):

        amount = 5
        results = 10
        my_test_photo_item = TEST_PHOTO_ITEM.copy()
        my_test_photo_item["like_count"] = self.bot.min_likes_to_like + 1
        my_test_photo_items = []
        for _ in range(results):
            my_test_photo_items.append(my_test_photo_item.copy())
        expect_filtered = 0
        my_test_photo_items[1]["has_liked"] = True
        expect_filtered += 1
        my_test_photo_items[2]["like_count"] = self.bot.max_likes_to_like + 1
        expect_filtered += 1
        my_test_photo_items[3]["like_count"] = self.bot.min_likes_to_like - 1
        expect_filtered += 1
        response_data = {
            "auto_load_more_enabled": True,
            "num_results": results,
            "status": "ok",
            "more_available": True,
            "next_max_id": TEST_PHOTO_ITEM['id'],
            "items": my_test_photo_items
        }

        responses.add(
            responses.GET, '{api_url}feed/tag/{hashtag}/?max_id={max_id}&rank_token={rank_token}&ranked_content=true&'.format(
                api_url=API_URL, hashtag=hashtag, max_id='',
                rank_token=self.bot.api.rank_token),
            json=response_data, status=200)

        medias = self.bot.get_total_hashtag_medias(hashtag, amount=amount, filtration=False)
        assert medias == [test_photo_item["pk"] for test_photo_item in my_test_photo_items[:amount]]
        assert len(medias) == amount

        medias = self.bot.get_total_hashtag_medias(hashtag, amount=amount, filtration=True)
        assert medias == [test_photo_item["pk"] for test_photo_item in my_test_photo_items[:amount] if (not test_photo_item["has_liked"] and test_photo_item["like_count"] < self.bot.max_likes_to_like and test_photo_item["like_count"] > self.bot.min_likes_to_like)]
        assert len(medias) == amount - expect_filtered

    @responses.activate
    @pytest.mark.parametrize('media_id', [
        '1234567890', 1234567890
    ])
    def test_get_media_likers(self, media_id):
        results = 5
        responses.add(
            responses.GET, "{api_url}media/{media_id}/likers/?".format(
                api_url=API_URL, media_id=media_id),
            json={
                "user_count": results,
                "status": "ok",
                "users": [TEST_MEDIA_LIKER for _ in range(results)]
            }, status=200)

        medias = self.bot.get_media_likers(media_id)

        assert medias == [str(TEST_MEDIA_LIKER["pk"]) for _ in range(results)]
        assert len(medias) == results

    @responses.activate
    @pytest.mark.parametrize('user_id', [
        19, '19'
    ])
    def test_get_last_user_medias(self, user_id):

        results = 5
        response_data = {
            "auto_load_more_enabled": True,
            "num_results": results,
            "status": "ok",
            "more_available": False,
            "items": [TEST_PHOTO_ITEM for _ in range(results)]
        }

        responses.add(
            responses.GET, '{api_url}feed/user/{user_id}/?max_id={max_id}&min_timestamp={min_timestamp}&rank_token={rank_token}&ranked_content=true'.format(
                api_url=API_URL, user_id=user_id, max_id='',
                min_timestamp=None, rank_token=self.bot.api.rank_token),
            json=response_data, status=200)

        medias = self.bot.get_last_user_medias(user_id, count=results)

        assert medias == [TEST_PHOTO_ITEM["pk"] for _ in range(results)]
        assert len(medias) == results

    @responses.activate
    @pytest.mark.parametrize('user_id', [
        19, '19'
    ])
    def test_get_total_user_medias(self, user_id):

        results = 18
        response_data = {
            "auto_load_more_enabled": True,
            "num_results": results,
            "status": "ok",
            "more_available": False,
            "items": [TEST_PHOTO_ITEM for _ in range(results)]
        }

        responses.add(
            responses.GET, '{api_url}feed/user/{user_id}/?max_id={max_id}&min_timestamp={min_timestamp}&rank_token={rank_token}&ranked_content=true'.format(
                api_url=API_URL, user_id=user_id, max_id='',
                min_timestamp=None, rank_token=self.bot.api.rank_token),
            json=response_data, status=200)

        medias = self.bot.get_total_user_medias(user_id)
        assert medias == [TEST_PHOTO_ITEM["pk"] for _ in range(results)]
        assert len(medias) == results

    @responses.activate
    @pytest.mark.parametrize('user_id', [
        '1234567890', 1234567890
    ])
    def test_get_user_likers(self, user_id):

        results_1 = 1
        responses.add(
            responses.GET, "{api_url}feed/user/{user_id}/?max_id={max_id}&min_timestamp={min_timestamp}&rank_token={rank_token}&ranked_content=true".format(
                api_url=API_URL, user_id=user_id, max_id='',
                min_timestamp=None, rank_token=self.bot.api.rank_token),
            json={
                "auto_load_more_enabled": True,
                "num_results": results_1,
                "status": "ok",
                "more_available": False,
                "items": [TEST_PHOTO_ITEM for _ in range(results_1)]
            }, status=200)

        results_2 = 3
        responses.add(
            responses.GET, "{api_url}media/{media_id}/likers/?".format(
                api_url=API_URL, media_id=TEST_PHOTO_ITEM['pk']),
            json={
                "user_count": results_2,
                "status": "ok",
                "users": [TEST_MEDIA_LIKER for _ in range(results_2)]
            }, status=200)

        user_ids = self.bot.get_user_likers(user_id)

        assert user_ids == list(set([str(TEST_MEDIA_LIKER["pk"]) for _ in range(results_2)]))
        assert len(user_ids) == len(list(set([str(TEST_MEDIA_LIKER["pk"]) for _ in range(results_2)])))

    @responses.activate
    @pytest.mark.parametrize('username', [
        '1234567890', 1234567890
    ])
    def test_get_user_followers(self, username):

        test_username = 'test.username'

        response_data_1 = {
            'status': 'ok',
            'user': TEST_SEARCH_USERNAME_ITEM
        }
        responses.add(
            responses.GET, '{api_url}users/{username}/usernameinfo/'.format(
                api_url=API_URL, username=test_username
            ), status=200, json=response_data_1)

        response_data_2 = {
            'status': 'ok',
            'user': TEST_USERNAME_INFO_ITEM
        }
        responses.add(
            responses.GET, '{api_url}users/{user_id}/info/'.format(
                api_url=API_URL, user_id=username
            ), status=200, json=response_data_2)

        results_3 = 5
        response_data_3 = {
            'status': 'ok',
            'big_list': False,
            'next_max_id': None,
            'sections': None,
            'users': [TEST_FOLLOWER_ITEM for _ in range(results_3)]
        }
        responses.add(
            responses.GET, "{api_url}friendships/{user_id}/followers/?rank_token={rank_token}".format(
                api_url=API_URL, user_id=username, rank_token=self.bot.api.rank_token
            ), json=response_data_3, status=200)

        user_ids = self.bot.get_user_followers(username)

        assert user_ids == [str(TEST_FOLLOWER_ITEM['pk']) for _ in range(results_3)]

    @responses.activate
    @pytest.mark.parametrize('username', [
        '1234567890', 1234567890
    ])
    def test_get_user_following(self, username):

        test_username = 'test.username'

        response_data_1 = {
            'status': 'ok',
            'user': TEST_SEARCH_USERNAME_ITEM
        }
        responses.add(
            responses.GET, '{api_url}users/{username}/usernameinfo/'.format(
                api_url=API_URL, username=test_username
            ), status=200, json=response_data_1)

        response_data_2 = {
            'status': 'ok',
            'user': TEST_USERNAME_INFO_ITEM
        }
        responses.add(
            responses.GET, '{api_url}users/{user_id}/info/'.format(
                api_url=API_URL, user_id=username
            ), status=200, json=response_data_2)

        results_3 = 5
        response_data_3 = {
            'status': 'ok',
            'big_list': False,
            'next_max_id': None,
            'sections': None,
            'users': [TEST_FOLLOWING_ITEM for _ in range(results_3)]
        }
        responses.add(
            responses.GET, "{api_url}friendships/{user_id}/following/?max_id={max_id}&ig_sig_key_version={sig_key}&rank_token={rank_token}".format(
                api_url=API_URL, user_id=username, rank_token=self.bot.api.rank_token, sig_key=SIG_KEY_VERSION, max_id=''
            ), json=response_data_3, status=200)

        user_ids = self.bot.get_user_following(username)

        assert user_ids == [str(TEST_FOLLOWING_ITEM['pk']) for _ in range(results_3)]

    @responses.activate
    @pytest.mark.parametrize('hashtag', [
        'hashtag1', 'hashtag2'
    ])
    def test_get_hashtag_users(self, hashtag):

        results = 5
        my_test_photo_item = TEST_PHOTO_ITEM.copy()
        my_test_photo_item["like_count"] = self.bot.min_likes_to_like + 1
        my_test_photo_items = []
        for _ in range(results):
            my_test_photo_items.append(my_test_photo_item.copy())
        expect_filtered = 0
        my_test_photo_items[1]["has_liked"] = True
        expect_filtered += 1
        my_test_photo_items[2]["like_count"] = self.bot.max_likes_to_like + 1
        expect_filtered += 1
        my_test_photo_items[3]["like_count"] = self.bot.min_likes_to_like - 1
        expect_filtered += 1
        response_data = {
            "auto_load_more_enabled": True,
            "num_results": results,
            "status": "ok",
            "more_available": False,
            "items": my_test_photo_items
        }

        responses.add(
            responses.GET, '{api_url}feed/tag/{hashtag}/?max_id={max_id}&rank_token={rank_token}&ranked_content=true&'.format(
                api_url=API_URL, hashtag=hashtag, max_id='',
                rank_token=self.bot.api.rank_token),
            json=response_data, status=200)

        medias = self.bot.get_hashtag_users(hashtag)
        assert medias == [str(test_photo_item["user"]["pk"]) for test_photo_item in my_test_photo_items]
        assert len(medias) == results

    @responses.activate
    @pytest.mark.parametrize('comment_id', [
        '12345678901234567', 12345678901234567
    ])
    def test_get_comment_likers(self, comment_id):
        results = 5
        response_data = {
            'status': 'ok',
            'users': [TEST_COMMENT_LIKER_ITEM for _ in range(results)]
        }
        responses.add(
            responses.GET, '{api_url}media/{comment_id}/comment_likers/?'.format(
                api_url=API_URL, comment_id=comment_id),
            json=response_data, status=200)
        user_ids = self.bot.get_comment_likers(comment_id)
        assert user_ids == [str(TEST_COMMENT_LIKER_ITEM['pk']) for _ in range(results)]
        assert len(user_ids) == results

    @responses.activate
    @pytest.mark.parametrize('latitude', [1.2345])
    @pytest.mark.parametrize('longitude', [9.8765])
    def test_get_locations_from_coordinates(self, latitude, longitude):
        results = 10
        response_data = {
            'has_more': False,
            'items': [TEST_LOCATION_ITEM for _ in range(results)],
            'rank_token': self.bot.api.rank_token,
            'status': 'ok'
        }
        responses.add(
            responses.GET, '{api_url}fbsearch/places/?rank_token={rank_token}&query={query}&lat={lat}&lng={lng}'.format(
                api_url=API_URL, rank_token=self.bot.api.rank_token, query='', lat=latitude, lng=longitude),
            json=response_data, status=200)
        locations = self.bot.get_locations_from_coordinates(latitude, longitude)
        assert locations == [TEST_LOCATION_ITEM for _ in range(results)]
        assert len(locations) == results

    @responses.activate
    def test_get_messages(self):
        results = 5
        response_data = {u'status': 'ok',
                         u'pending_requests_total': 2,
                         u'seq_id': 182,
                         u'snapshot_at_ms': 1547815538244,
                         u'most_recent_inviter': TEST_MOST_RECENT_INVITER_ITEM,
                         u'inbox': {u'blended_inbox_enabled': True,
                                    u'has_older': False,
                                    u'unseen_count': 1,
                                    u'unseen_count_ts': 1547815538242025,
                                    u'threads': [TEST_INBOX_THREAD_ITEM for _ in range(results)]
                                    }
                         }
        responses.add(
            responses.GET, '{api_url}direct_v2/inbox/?'.format(
                api_url=API_URL), json=response_data, status=200)
        inbox = self.bot.get_messages()
        assert inbox == response_data



from tqdm import tqdm


def unfollow(self, user_id):
    user_id = self.convert_to_user_id(user_id)
    user_info = self.get_user_info(user_id)
    username = user_info.get("username")
    self.console_print('===> Going to unfollow `user_id`: {} with username: {}'.format(user_id, username))

    if self.check_user(user_id, unfollowing=True):
        return True  # whitelisted user
    if not self.reached_limit('unfollows'):
        self.delay('unfollow')
        if self.api.unfollow(user_id):
            msg = '===> Unfollowed, `user_id`: {}, user_name: {}'
            self.console_print(msg.format(user_id, username), 'yellow')
            self.unfollowed_file.append(user_id)
            self.total['unfollows'] += 1
            if user_id in self.following:
                self.following.remove(user_id)
            return True
    else:
        self.logger.info("Out of unfollows for today.")
    return False


def unfollow_users(self, user_ids):
    broken_items = []
    self.logger.info("Going to unfollow {} users.".format(len(user_ids)))
    user_ids = set(map(str, user_ids))
    filtered_user_ids = list(set(user_ids) - set(self.whitelist))
    if len(filtered_user_ids) != len(user_ids):
        self.logger.info(
            "After filtration by whitelist {} users left.".format(len(filtered_user_ids)))
    for user_id in tqdm(filtered_user_ids, desc='Processed users'):
        if not self.unfollow(user_id):
            self.error_delay()
            i = filtered_user_ids.index(user_id)
            broken_items = filtered_user_ids[i:]
            break
    self.logger.info("DONE: Total unfollowed {} users.".format(self.total['unfollows']))
    return broken_items


def unfollow_non_followers(self, n_to_unfollows=None):
    self.logger.info("Unfollowing non-followers.")
    self.console_print(" ===> Start unfollowing non-followers <===", 'red')
    non_followers = set(self.following) - set(self.followers) - self.friends_file.set
    non_followers = list(non_followers)
    for user_id in tqdm(non_followers[:n_to_unfollows]):
        if self.reached_limit('unfollows'):
            self.logger.info("Out of unfollows for today.")
            break
        self.unfollow(user_id)
    self.console_print(" ===> Unfollow non-followers done! <===", 'red')


def unfollow_everyone(self):
    self.unfollow_users(self.following)



from __future__ import unicode_literals
import os
import shutil
import time
from random import randint
from requests_toolbelt import MultipartEncoder
import json

from . import config
from .api_photo import resize_image, compatible_aspect_ratio, get_image_size


def download_story(self, filename, story_url, username):
    path = "stories/{}".format(username)
    if not os.path.exists(path):
        os.makedirs(path)
    fname = os.path.join(path, filename)
    if os.path.exists(fname):  # already exists
        self.logger.info("Stories already downloaded...")
        return os.path.abspath(fname)
    response = self.session.get(story_url, stream=True)
    if response.status_code == 200:
        with open(fname, 'wb') as f:
            response.raw.decode_content = True
            shutil.copyfileobj(response.raw, f)
        return os.path.abspath(fname)


def upload_story_photo(self, photo, upload_id=None):
    if upload_id is None:
        upload_id = str(int(time.time() * 1000))
    photo = resize_image(photo)
    if not photo:
        return False
    if not compatible_aspect_ratio(get_image_size(photo)):
        self.logger.error('Photo does not have a compatible photo aspect ratio.')
        return False

    with open(photo, 'rb') as f:
        photo_bytes = f.read()

    data = {
        'upload_id': upload_id,
        '_uuid': self.uuid,
        '_csrftoken': self.token,
        'image_compression': '{"lib_name":"jt","lib_version":"1.3.0","quality":"87"}',
        'photo': ('pending_media_%s.jpg' % upload_id, photo_bytes, 'application/octet-stream', {'Content-Transfer-Encoding': 'binary'})
    }
    m = MultipartEncoder(data, boundary=self.uuid)
    self.session.headers.update({'X-IG-Capabilities': '3Q4=',
                                 'X-IG-Connection-Type': 'WIFI',
                                 'Cookie2': '$Version=1',
                                 'Accept-Language': 'en-US',
                                 'Accept-Encoding': 'gzip, deflate',
                                 'Content-type': m.content_type,
                                 'Connection': 'close',
                                 'User-Agent': self.user_agent})
    response = self.session.post(
        config.API_URL + "upload/photo/", data=m.to_string())

    if response.status_code == 200:
        upload_id = json.loads(response.text).get('upload_id')
        if self.configure_story(upload_id, photo):
            # self.expose()
            return True
    return False


def configure_story(self, upload_id, photo):
    (w, h) = get_image_size(photo)
    data = self.json_data({
        'source_type': 4,
        'upload_id': upload_id,
        'story_media_creation_date': str(int(time.time()) - randint(11, 20)),
        'client_shared_at': str(int(time.time()) - randint(3, 10)),
        'client_timestamp': str(int(time.time())),
        'configure_mode': 1,      # 1 - REEL_SHARE, 2 - DIRECT_STORY_SHARE
        'device': self.device_settings,
        'edits': {
            'crop_original_size': [w * 1.0, h * 1.0],
            'crop_center': [0.0, 0.0],
            'crop_zoom': 1.3333334
        },
        'extra': {
            'source_width': w,
            'source_height': h,
        }})
    return self.send_request('media/configure_to_story/?', data)



def download_stories(self, username):
    user_id = self.get_user_id_from_username(username)
    list_image, list_video = self.get_user_stories(user_id)
    if list_image == [] and list_video == []:
        self.logger.error(
            "Make sure that '{}' is NOT private and that posted some stories".format(username))
        return False
    self.logger.info("Downloading stories...")
    for story_url in list_image:
        filename = story_url.split('/')[-1].split('.')[0] + ".jpg"
        self.api.download_story(filename, story_url, username)
    for story_url in list_video:
        filename = story_url.split('/')[-1].split('.')[0] + ".mp4"
        self.api.download_story(filename, story_url, username)


def upload_story_photo(self, photo, upload_id=None):
    self.small_delay()
    if self.api.upload_story_photo(photo, upload_id):
        self.logger.info("Photo '{}' is uploaded as Story.".format(photo))
        return True
    self.logger.info("Photo '{}' is not uploaded.".format(photo))
    return False



from __future__ import unicode_literals
import imghdr
import os
import shutil
import struct
import time

from requests_toolbelt import MultipartEncoder

from . import config


def download_photo(self, media_id, filename, media=False, folder='photos'):
    if not media:
        self.media_info(media_id)
        if not self.last_json.get('items'):
            return True
        media = self.last_json['items'][0]
    if media['media_type'] == 2:
        return True
    elif media['media_type'] == 1:
        filename = ('{}_{}.jpg'.format(media['user']['username'], media_id)
                    if not filename else '{}.jpg'.format(filename))
        images = media['image_versions2']['candidates']
        fname = os.path.join(folder, filename)
        if os.path.exists(fname):
            self.logger.info("File already esists, skipping...")
            return os.path.abspath(fname)
        response = self.session.get(images[0]['url'], stream=True)
        if response.status_code == 200:
            with open(fname, 'wb') as f:
                response.raw.decode_content = True
                shutil.copyfileobj(response.raw, f)
            return os.path.abspath(fname)
    else:
        success = False
        video_included = False
        for index in range(len(media["carousel_media"])):
            if media["carousel_media"][index]["media_type"] != 1:
                video_included = True
                continue
            filename_i = ('{}_{}_{}.jpg'.format(media['user']['username'], media_id, index)
                          if not filename else '{}_{}.jpg'.format(filename, index))
            images = media["carousel_media"][index]["image_versions2"]["candidates"]
            fname = os.path.join(folder, filename_i)
            if os.path.exists(fname):
                return os.path.abspath(fname)
            response = self.session.get(images[0]['url'], stream=True)
            if response.status_code == 200:
                success = True
                with open(fname, 'wb') as f:
                    response.raw.decode_content = True
                    shutil.copyfileobj(response.raw, f)
        if success:
            return os.path.abspath(fname)
        elif video_included:
            return True


def compatible_aspect_ratio(size):
    min_ratio, max_ratio = 4.0 / 5.0, 90.0 / 47.0
    width, height = size
    ratio = width * 1. / height * 1.
    print("FOUND: w:{} h:{} r:{}".format(width, height, ratio))
    return min_ratio <= ratio <= max_ratio


def configure_photo(self, upload_id, photo, caption=''):
    (w, h) = get_image_size(photo)
    data = self.json_data({
        'media_folder': 'Instagram',
        'source_type': 4,
        'caption': caption,
        'upload_id': upload_id,
        'device': self.device_settings,
        'edits': {
            'crop_original_size': [w * 1.0, h * 1.0],
            'crop_center': [0.0, 0.0],
            'crop_zoom': 1.0
        },
        'extra': {
            'source_width': w,
            'source_height': h,
        }})
    return self.send_request('media/configure/?', data)


def upload_photo(self, photo, caption=None, upload_id=None, from_video=False,
                 force_rezize=False):
    if upload_id is None:
        upload_id = str(int(time.time() * 1000))
    if not photo:
        return False
    if not compatible_aspect_ratio(get_image_size(photo)):
        self.logger.error('Photo does not have a compatible photo aspect ratio.')
        if force_rezize:
            photo = resize_image(photo)
        else:
            return False

    with open(photo, 'rb') as f:
        photo_bytes = f.read()

    data = {
        'upload_id': upload_id,
        '_uuid': self.uuid,
        '_csrftoken': self.token,
        'image_compression': '{"lib_name":"jt","lib_version":"1.3.0","quality":"87"}',
        'photo': ('pending_media_%s.jpg' % upload_id, photo_bytes, 'application/octet-stream', {'Content-Transfer-Encoding': 'binary'})
    }
    m = MultipartEncoder(data, boundary=self.uuid)
    self.session.headers.update({'X-IG-Capabilities': '3Q4=',
                                 'X-IG-Connection-Type': 'WIFI',
                                 'Cookie2': '$Version=1',
                                 'Accept-Language': 'en-US',
                                 'Accept-Encoding': 'gzip, deflate',
                                 'Content-type': m.content_type,
                                 'Connection': 'close',
                                 'User-Agent': self.user_agent})
    response = self.session.post(
        config.API_URL + "upload/photo/", data=m.to_string())

    if response.status_code == 200:
        if self.configure_photo(upload_id, photo, caption):
            self.expose()
            from os import rename
            rename(photo, "{}.REMOVE_ME".format(photo))
            return True
    return False


def get_image_size(fname):
    with open(fname, 'rb') as fhandle:
        head = fhandle.read(24)
        if len(head) != 24:
            raise RuntimeError("Invalid Header")

        if imghdr.what(fname) == 'png':
            check = struct.unpack('>i', head[4:8])[0]
            if check != 0x0d0a1a0a:
                raise RuntimeError("PNG: Invalid check")
            width, height = struct.unpack('>ii', head[16:24])
        elif imghdr.what(fname) == 'gif':
            width, height = struct.unpack('<HH', head[6:10])
        elif imghdr.what(fname) == 'jpeg':
            fhandle.seek(0)  # Read 0xff next
            size = 2
            ftype = 0
            while not 0xc0 <= ftype <= 0xcf:
                fhandle.seek(size, 1)
                byte = fhandle.read(1)
                while ord(byte) == 0xff:
                    byte = fhandle.read(1)
                ftype = ord(byte)
                size = struct.unpack('>H', fhandle.read(2))[0] - 2
            # We are at a SOFn block
            fhandle.seek(1, 1)  # Skip `precision' byte.
            height, width = struct.unpack('>HH', fhandle.read(4))
        else:
            raise RuntimeError("Unsupported format")
        return width, height


def resize_image(fname):
    from math import ceil
    try:
        from PIL import Image, ExifTags
    except ImportError as e:
        print("ERROR: {}".format(e))
        print("Required module `PIL` not installed\n"
              "Install with `pip install Pillow` and retry")
        return False
    print("Analizing `{}`".format(fname))
    h_lim = {'w': 90., 'h': 47.}
    v_lim = {'w': 4., 'h': 5.}
    img = Image.open(fname)
    (w, h) = img.size
    deg = 0
    try:
        for orientation in ExifTags.TAGS.keys():
            if ExifTags.TAGS[orientation] == 'Orientation':
                break
        exif = dict(img._getexif().items())
        o = exif[orientation]
        if o == 3:
            deg = 180
        if o == 6:
            deg = 270
        if o == 8:
            deg = 90
        if deg != 0:
            print("Rotating by {d} degrees".format(d=deg))
            img = img.rotate(deg, expand=True)
            (w, h) = img.size
    except (AttributeError, KeyError, IndexError) as e:
        print("No exif info found (ERR: {})".format(e))
        pass
    img = img.convert("RGBA")
    ratio = w * 1. / h * 1.
    print("FOUND w:{w}, h:{h}, ratio={r}".format(w=w, h=h, r=ratio))
    if w > h:
        print("Horizontal image")
        if ratio > (h_lim['w'] / h_lim['h']):
            print("Cropping image")
            cut = int(ceil((w - h * h_lim['w'] / h_lim['h']) / 2))
            left = cut
            right = w - cut
            top = 0
            bottom = h
            img = img.crop((left, top, right, bottom))
            (w, h) = img.size
        if w > 1080:
            print("Resizing image")
            nw = 1080
            nh = int(ceil(1080. * h / w))
            img = img.resize((nw, nh), Image.ANTIALIAS)
    elif w < h:
        print("Vertical image")
        if ratio < (v_lim['w'] / v_lim['h']):
            print("Cropping image")
            cut = int(ceil((h - w * v_lim['h'] / v_lim['w']) / 2))
            left = 0
            right = w
            top = cut
            bottom = h - cut
            img = img.crop((left, top, right, bottom))
            (w, h) = img.size
        if h > 1080:
            print("Resizing image")
            nw = int(ceil(1080. * w / h))
            nh = 1080
            img = img.resize((nw, nh), Image.ANTIALIAS)
    else:
        print("Square image")
        if w > 1080:
            print("Resizing image")
            img = img.resize((1080, 1080), Image.ANTIALIAS)
    (w, h) = img.size
    new_fname = "{}.CONVERTED.jpg".format(fname)
    print("Saving new image w:{w} h:{h} to `{f}`".format(w=w, h=h, f=new_fname))
    new = Image.new("RGB", img.size, (255, 255, 255))
    new.paste(img, (0, 0, w, h), img)
    new.save(new_fname, quality=95)
    return new_fname




import pytest
import responses

try:
    from unittest.mock import patch
except ImportError:
    from mock import patch

from instabot.api.config import API_URL

from .test_bot import TestBot
from .test_variables import (TEST_CAPTION_ITEM, TEST_COMMENT_ITEM, TEST_PHOTO_ITEM,
                             TEST_USERNAME_INFO_ITEM)


class TestBotFilter(TestBot):

    @responses.activate
    @pytest.mark.parametrize('media_id,total,max_per_day', [
        [111111, 1, 2],
        [111111, 2, 2],
        [111111, 3, 2],
        ['111111', 1, 2],
        ['111111', 2, 2],
        ['111111', 3, 2]])
    @patch('time.sleep', return_value=None)
    def test_unlike(self, patched_time_sleep, media_id, total, max_per_day):
        self.bot.total['unlikes'] = total
        self.bot.max_per_day['unlikes'] = max_per_day
        responses.add(
            responses.POST, "{api_url}media/{media_id}/unlike/".format(
                api_url=API_URL, media_id=media_id
            ), json="{'status': 'ok'}", status=200
        )
        _r = self.bot.unlike(media_id)
        test_r = _r if total < max_per_day else not _r
        test_unliked = self.bot.total['unlikes'] == total + 1 if total < max_per_day else self.bot.total['unlikes'] == total
        assert (test_r and test_unliked)

    @responses.activate
    @pytest.mark.parametrize('comment_id', [111111, '111111'])
    @patch('time.sleep', return_value=None)
    def test_unlike_comment(self, patched_time_sleep, comment_id):
        responses.add(
            responses.POST, "{api_url}media/{comment_id}/comment_unlike/".format(
                api_url=API_URL, comment_id=comment_id
            ), json="{'status': 'ok'}", status=200
        )
        _r = self.bot.unlike_comment(comment_id)
        assert _r

    @responses.activate
    @pytest.mark.parametrize('media_ids,total,max_per_day', [
        [[111111, 222222], 1, 3],
        [[111111, 222222], 2, 3],
        [[111111, 222222], 3, 3],
        [['111111', '222222'], 1, 3],
        [['111111', '222222'], 2, 3],
        [['111111', '222222'], 3, 3]])
    @patch('time.sleep', return_value=None)
    def test_unlike_medias(self, patched_time_sleep, media_ids, total, max_per_day):
        self.bot.total['unlikes'] = total
        self.bot.max_per_day['unlikes'] = max_per_day
        for media_id in media_ids:
            responses.add(
                responses.POST, "{api_url}media/{media_id}/unlike/".format(
                    api_url=API_URL, media_id=media_id
                ), json="{'status': 'ok'}", status=200
            )
        broken_items = self.bot.unlike_medias(media_ids)
        test_unliked = self.bot.total['unlikes'] == max_per_day
        test_broken = len(broken_items) == len(media_ids) - (max_per_day - total)
        assert (test_unliked and test_broken)

    @responses.activate
    @patch('time.sleep', return_value=None)
    def test_unlike_media_comments(self, patched_time_sleep):
        my_test_comment_items = []
        results = 5
        for i in range(results):
            my_test_comment_items.append(TEST_COMMENT_ITEM.copy())
            my_test_comment_items[i]['pk'] = TEST_COMMENT_ITEM['pk'] + i
            if i % 2:
                my_test_comment_items[i]['has_liked_comment'] = False
            else:
                my_test_comment_items[i]['has_liked_comment'] = True
        media_id = 1234567890
        response_data = {
            "caption": TEST_CAPTION_ITEM,
            "caption_is_edited": False,
            "comment_count": results,
            "comment_likes_enabled": True,
            "comments": my_test_comment_items,
            "has_more_comments": False,
            "has_more_headload_comments": False,
            "media_header_display": "none",
            "preview_comments": [],
            "status": "ok"
        }
        responses.add(
            responses.GET, '{api_url}media/{media_id}/comments/?'.format(
                api_url=API_URL, media_id=media_id), json=response_data, status=200)
        for my_test_comment_item in my_test_comment_items:
            responses.add(
                responses.POST, "{api_url}media/{comment_id}/comment_unlike/".format(
                    api_url=API_URL, comment_id=my_test_comment_item['pk']
                ), json="{'status': 'ok'}", status=200
            )
        broken_items = self.bot.unlike_media_comments(media_id)
        assert broken_items == []

    @responses.activate
    @patch('time.sleep', return_value=None)
    def test_unlike_user(self, patched_time_sleep):
        unliked_at_start = self.bot.total['unlikes']
        user_id = 1234567890
        response_data = {
            'status': 'ok',
            'user': TEST_USERNAME_INFO_ITEM
        }
        responses.add(
            responses.GET, '{api_url}users/{username}/usernameinfo/'.format(
                api_url=API_URL, username=user_id
            ), status=200, json=response_data)
        my_test_photo_items = []
        results = 5
        for i in range(results):
            my_test_photo_items.append(TEST_PHOTO_ITEM.copy())
            my_test_photo_items[i]['pk'] = TEST_PHOTO_ITEM['pk'] + i
            if i % 2:
                my_test_photo_items[i]['has_liked'] = False
            else:
                my_test_photo_items[i]['has_liked'] = True
        response_data = {
            "auto_load_more_enabled": True,
            "num_results": results,
            "status": "ok",
            "more_available": False,
            "items": my_test_photo_items
        }
        responses.add(
            responses.GET, '{api_url}feed/user/{user_id}/'.format(
                api_url=API_URL, user_id=user_id),
            json=response_data, status=200)
        for my_test_photo_item in my_test_photo_items:
            responses.add(
                responses.POST, "{api_url}media/{media_id}/unlike/".format(
                    api_url=API_URL, media_id=my_test_photo_item['pk']
                ), json="{'status': 'ok'}", status=200
            )
        broken_items = self.bot.unlike_user(user_id)
        test_unliked = self.bot.total['unlikes'] == unliked_at_start + len(my_test_photo_items)
        test_broken = broken_items == []
        assert (test_broken and test_unliked)
